package com.mylaensys.dsl.generator

import com.mylaensys.dsl.gape.ModuleDeclaration
import org.eclipse.xtext.generator.IFileSystemAccess
import java.util.HashSet
import com.mylaensys.dsl.gape.MethodDeclaration

class CodeGenerator {
	
	def void compile(ModuleDeclaration module, IFileSystemAccess fsa) {
		/* pre compilation */
		val symbolTable = new SymbolTable		
		for(object : module.objects) {
			/* Push declared interaction in the Symbol Table */			
			symbolTable.objects.put( module.name + "." + object.name, object )
			for( m : object.methods) {
				for(i : m.interactions) {
					var iname = object.name + "$" + i 
					if( symbolTable.interactionInterfaces.containsKey(iname) ) {
						System.out.println("Adding method " + m.name + " to " + iname  )
						symbolTable.interactionInterfaces.get(iname).add( m )
					} else {
						System.out.println("Creating " + iname + " for method " + m.name)
						var set = new HashSet<MethodDeclaration>()
						set.add( m )
						symbolTable.interactionInterfaces.put( iname , set) 
					}
				}
				
			}
		}
		for(s : symbolTable.objects.keySet ) {
			System.out.println("Symbol: " + s  + " " + symbolTable.objects.get(s) )
		}
		
		/* registry  */
		val registry = new RegistryGenerator
		registry.compile(module,symbolTable,fsa)
		
		/* compilation */
		for(object : module.objects) {
			
			val CompilationUnit unit = CodeGeneratorFactory.createModelGenerator(module.target);
			unit.modelGenerator.compile( module , object , symbolTable ,fsa )
			unit.serviceGenerator.compile( module , object , symbolTable, fsa )
		}
		
		
		val doc = new DocumentationGenerator
		doc.compile( module,symbolTable,fsa )
	}	
}