package com.mylaensys.dsl.generator

import com.mylaensys.dsl.gape.ModuleDeclaration
import org.eclipse.xtext.generator.IFileSystemAccess
import com.mylaensys.dsl.gape.ObjectDeclaration
import com.mylaensys.dsl.generator.utils.Utils

class DocumentationGenerator {
	
	def compile(ModuleDeclaration module,SymbolTable symbolTable,IFileSystemAccess fsa) {
		fsa.generateFile("/" + module.name + ".html", compileDocumentation(module,symbolTable));
	}
	
	def compileDocumentation(ModuleDeclaration module,SymbolTable table)'''
		<!DOCTYPE html>
		<html>
		<head>
		    <title>«module.name» - Ver. «module.version»</title>
		</head>
		<body>

		«FOR key : table.objects.keySet»
			«val ObjectDeclaration object = table.objects.get(key)»
			<h2>Get «Utils.makePlural(object.name.toFirstUpper)»</h2>
			<hr/>
			<p>Returns the list of available «Utils.makePlural(object.name.toLowerCase)».</p>
			<p>
				<pre>GET - /«module.version»/«Utils.makePlural(object.name.toLowerCase)»</pre>
			</p>	
			<p>			
				<b>Request Parameters</b>
				<p>None</p>
			</p>
			<p>
				<b>Response</b>
				<ul>
					<li> <strong>200</strong> - Succeeded </li>
				</ul>
			</p>	
			<br/>
			<p>	
				<b>Example</b>
				<p></p>
			</p>
			
			«/* Get */»
			<h2>Get «object.name.toFirstUpper»</h2>
			<hr/>
			<p>Returns the «object.name.toFirstUpper» details.</p>
			<p>
				<pre>GET - /«module.version»/«Utils.makePlural(object.name.toLowerCase)»/{id}</pre>
			</p>	
			<p>			
				<b>Request Parameters</b>
				<ul>
					<li> <strong>{id}</strong> (url path paramter)</li>
				</ul>
			</p>
			<p>
				<b>Response</b>
				<ul>
					<li> <strong>200</strong> - Succeeded </li>
					<li> <strong>500</strong> - Not Found </li>
				</ul>
			</p>	
			<br/>
			<p>	
				<b>Example</b>
				<p></p>
			</p>	
			
			
			«/* Create */»
			<h2>Create «object.name.toFirstUpper»</h2>
			<hr/>
			<p>Creates a «object.name.toFirstUpper».</p>
			<p>
				<pre>POST - /«module.version»/«Utils.makePlural(object.name.toLowerCase)»</pre>
			</p>	
			<p>			
				<b>Request Parameters</b>
				<ul>
					«FOR p : object.properties»
					«IF !Utils.isSupportedCollection( p.typeOrCollection.name )»
					<li> <strong>«p.name.toLowerCase»</strong></li>
					«ENDIF»
					«ENDFOR»
				</ul>
			</p>
			<p>
				<b>Response</b>
				<ul>
					<li> <strong>200</strong> - Succeeded </li>
					<li> <strong>400</strong> - Invalid data</li>
				</ul>
			</p>	
			<br/>
			<p>	
				<b>Example</b>
				<p></p>
			</p>	
			
			«/* Update */»
			<h2>Update «object.name.toFirstUpper»</h2>
			<hr/>
			<p>Update a «object.name.toFirstUpper».</p>
			<p>
				<pre>POST - /«module.version»/«Utils.makePlural(object.name.toLowerCase)»/{id}</pre>
			</p>	
			<p>			
				<b>Request Parameters</b>

				<ul>
					<li> <strong>{id}</strong> (url path paramter)</li>
					«FOR p : object.properties»
					«IF !Utils.isSupportedCollection( p.typeOrCollection.name  )»
					<li> <strong>«p.name.toLowerCase»</strong></li>
					«ENDIF»
					«ENDFOR»
				</ul>
			</p>
			<p>
				<b>Response</b>
				<ul>
					<li> <strong>200</strong> - Succeeded </li>
					<li> <strong>400</strong> - Invalid data</li>
				</ul>
			</p>	
			<br/>
			<p>	
				<b>Example</b>
				<p></p>
			</p>	
			«FOR p : object.properties»
				«IF Utils.isSupportedCollection( p.typeOrCollection.name  )»
					«/* Get */»
					<h2>Get «object.name.toFirstUpper» «p.name.toFirstUpper»</h2>	
					<hr/>
					<p>Return the list of «Utils.makePlural(p.type.name.toFirstUpper)» in a «object.name».</p>
					<p>
						<pre>GET - /«module.version»/«Utils.makePlural(object.name.toLowerCase)»/{id}/«p.name.toLowerCase»</pre>
					</p>	
					<p>			
						<b>Request Parameters</b>
						<ul>
							<li> <strong>{id}</strong> (url path paramter)</li>
						</ul>
					</p>
					<p>
						<b>Response</b>
						<ul>
							<li> <strong>200</strong> - Succeeded </li>
							<li> <strong>500</strong> - Not Found </li>
						</ul>
					</p>	
					
					«/* Add */»
					<h2>Add «p.type.name.toFirstUpper» to «object.name.toFirstUpper» </h2>	
					<hr/>
					<p>Add a «p.type.name.toFirstUpper» to «object.name».</p>
					<p>
						<pre>POST - /«module.version»/«Utils.makePlural(object.name.toLowerCase)»/{id}/«p.name.toLowerCase»/{toAdd}</pre>
					</p>	
					<p>			
						<b>Request Parameters</b>
						<ul>
							<li> <strong>{id}</strong> (url path paramter)</li>
							<li> <strong>{toAdd}</strong> (url path paramter)</li>
						</ul>
					</p>
					<p>
						<b>Response</b>
						<ul>
							<li> <strong>200</strong> - Succeeded </li>
							<li> <strong>500</strong> - Not Found </li>
						</ul>
					</p>
					
					«/* Remove */»
					<h2>Remove «p.type.name.toFirstUpper» from «object.name.toFirstUpper» </h2>	
					<hr/>
					<p>Remove a «p.type.name.toFirstUpper» to «object.name».</p>
					<p>
						<pre>DELETE - /«module.version»/«Utils.makePlural(object.name.toLowerCase)»/{id}/«p.name.toLowerCase»/{toRemove}</pre>
					</p>	
					<p>			
						<b>Request Parameters</b>
						<ul>
							<li> <strong>{id}</strong> (url path paramter)</li>
							<li> <strong>{toRemove}</strong> (url path paramter)</li>
						</ul>
					</p>
					<p>
						<b>Response</b>
						<ul>
							<li> <strong>200</strong> - Succeeded </li>
							<li> <strong>500</strong> - Not Found </li>
						</ul>
					</p>	
						
				«ENDIF»
			«ENDFOR»
			
			 
		«ENDFOR»
		</body>
		</html>
		
	'''
	
}