package com.mylaensys.dsl.generator

import com.mylaensys.dsl.gape.ModuleDeclaration
import org.eclipse.xtext.generator.IFileSystemAccess
import org.apache.log4j.Logger
import org.eclipse.xtext.xbase.compiler.ImportManager
import com.mylaensys.dsl.generator.utils.Utils
import com.mylaensys.dsl.gape.PropertyDeclaration
import java.util.ArrayList
import org.eclipse.emf.common.util.EList
import com.mylaensys.dsl.gape.ObjectDeclaration

class RegistryGenerator {
	protected static Logger log =  Logger.getLogger("RegistryGenerator")
	private int index = 0;	
	protected SymbolTable symbolTable
	
	
	def compile(ModuleDeclaration module,SymbolTable symbolTable,IFileSystemAccess fsa) {
		this.symbolTable = symbolTable		
		fsa.generateFile(pathName(module.name) + "/" + "RegistryImpl.java", compileRegistry(module.name));
	}
	
	def compileRegistry(String targetPackage) '''
		«log.debug("compiling: registry for:" + targetPackage)»		
		«val importManager = new ImportManager(true,Utils.createJvmType( "Registry", targetPackage ))»
		«val body = compileRegistryBody(targetPackage,importManager)»
		package «targetPackage»;		
		«FOR i : importManager.imports»
			import «i»;
		«ENDFOR»
		«body»
	'''
	/* Compile the registry body */
	def compileRegistryBody(String targetPackage,ImportManager im)'''
	
	«val hashmap = Utils.createJvmType("HashMap","java.util")»
	«val object = Utils.createJvmType("RegistryObject","com.mylaensys.dsl.gape")»
	«val registry = Utils.createJvmType("Registry","com.mylaensys.dsl.gape")»
	/** Registry Singleton. */
	public final class RegistryImpl implements «im.serialize(registry)» {
		/** Entries. */
		private «im.serialize(hashmap)»<String, «im.serialize(object)»> objects;
		/** Default constructor. */
		private RegistryImpl() {
			this.objects = new «im.serialize(hashmap)»();
			«compileRegistryEntries(targetPackage,im)»
		}
		/** Lazy Holder. */
		private static class LazyHolder {
			private static final «im.serialize(registry)» INSTANCE = new RegistryImpl();
		}
		/** Registry Singleton. @return unique Registry instance */
		public static «im.serialize(registry)» getInstance() {
			return LazyHolder.INSTANCE;
		}
		/** Returns an object. 
		 * @param name object name
		 * @return unique Registry instance */
		@Override
		public RegistryObject getObject(final String name) {
			return this.objects.get(name);
		}
		
	}
	'''
	def compileRegistryEntries(String targetPackage,ImportManager im) '''
	«val clazz = Utils.createJvmType("RegistryObject","com.mylaensys.dsl.gape")»
	«val property = Utils.createJvmType("RegistryProperty","com.mylaensys.dsl.gape")»
	«FOR o : symbolTable.getObjects.keySet»
		«var object = symbolTable.getObjects.get(o)»
		/* «object.name» */
		«im.serialize(clazz)» «object.name.toLowerCase» = new «im.serialize(clazz)»();
		«var writables = getWriteableProperties(object.properties)»
		«FOR p : writables»			
			«IF symbolTable.objects.containsKey( targetPackage + "." + p.typeOrCollection.name)»
			/* «p.name» - reference */
			«im.serialize(property)» «object.name.toFirstLower»«p.name.toFirstUpper» = new «im.serialize(property)»(true);
			«object.name.toLowerCase».getProperties().put("«p.name»", «object.name.toFirstLower»«p.name.toFirstUpper»);
			«ELSE»	 
			/* «p.name» */
			«im.serialize(property)» «object.name.toFirstLower»«p.name.toFirstUpper» = new «im.serialize(property)»(false);
			«object.name.toLowerCase».getProperties().put("«p.name»", «object.name.toFirstLower»«p.name.toFirstUpper»);
			«ENDIF»
		«ENDFOR»
		this.objects.put("«o»", «object.name.toLowerCase»);
	«ENDFOR»
	'''

	/* Returns the list of writable properties */
	def getWriteableProperties(EList<PropertyDeclaration> properties) {
		var writable = new ArrayList<PropertyDeclaration>(); 
		for(p : properties) {
			if( isWritableProperty( p ) ) {
				writable.add( p )	
			}
		}
		return writable
	}
	
	def isWritableProperty(PropertyDeclaration property) {
		/* Collections are not writable */
		if( Utils.isSupportedCollection(property.typeOrCollection.name))
			return false
			
		/* Virtual properties are not writable */	
		if( "virtual".equals(property.virtual) ) 			
			return false
			
		/* if no permission is specified is writable */	
		if( property.permissions.empty ) 			
			return true
			
		/* if has write permission is writable */	
		for(p:property.permissions) {
			if( "write".equalsIgnoreCase( p.name ))
				return true
		}
		return false;
	}
	
	/* Computes the registry class name */
	def registryName(String className) {
		return className + "Interface"
	}
	
	/* compute the target path */	 
	def pathName(String pkg) {
		return pkg.replace(".","/")
	}
	/* compute the index */
	def getIndex() {
		this.index = this.index + 1;
		return index 
	}	
	
}