package com.mylaensys.dsl.generator.jdo

import com.mylaensys.dsl.generator.ModelGenerator
import com.mylaensys.dsl.gape.ObjectDeclaration
import org.eclipse.xtext.generator.IFileSystemAccess
import com.mylaensys.dsl.generator.pojo.ModelGeneratorPojo
import org.eclipse.xtext.xbase.compiler.ImportManager
import com.mylaensys.dsl.gape.PropertyDeclaration
import com.mylaensys.dsl.gape.ModuleDeclaration
import com.mylaensys.dsl.generator.SymbolTable
import org.apache.log4j.Logger
import org.apache.log4j.Level
import org.eclipse.emf.common.util.EList
import org.eclipse.xtext.common.types.JvmType
import com.mylaensys.dsl.generator.utils.Utils
import com.mylaensys.dsl.GapeOutputConfigurationProvider
import com.mylaensys.dsl.gape.InteractionDeclaration

class ModelGeneratorJDO extends ModelGeneratorPojo implements ModelGenerator {
	 		
	override compile(ModuleDeclaration module,ObjectDeclaration object,SymbolTable symbolTable,IFileSystemAccess fsa) {
		log.setLevel( Level.DEBUG )								
		/* Compile the Interface */
		this.symbolTable = symbolTable
		fsa.generateFile(pathName(module.name) + "/" + interfaceName(object.name) + ".java", compileInterface(object,module.name));		
		/* Compile the Abstract Class */
		fsa.generateFile(pathName(module.name) + "/" + abstractClassName(object.name) + ".java", GapeOutputConfigurationProvider::GEN_ONCE_OUTPUT, compileAbstractClass(object,module.name));
		/* Compile the Domain Class */
		fsa.generateFile(pathName(module.name) + "/" + object.name + ".java", compileDomainClass(object,module.name));
		/* Compile the DTO */
		fsa.generateFile(pathName(module.name) + "/" + dtoClassName(object.name) + ".java", compileDataTransferObjectClass(object,module.name));
		/* Compile the Contract Interface */
		fsa.generateFile(pathName(module.name) + "/" + contractClassName(object.name) + ".java", compileContractInterface(object,module.name));
		/* Compile the Interaction Interfaces */		
		for(interaction : symbolTable.interactionInterfaces.keySet) {
			fsa.generateFile(pathName(module.name) + "/" + interaction + ".java", compileInteractionInterface(object,interaction,module.name));
		}
		for(interaction : object.interactions ) {
			fsa.generateFile(pathName(module.name) + "/" + object.name+interaction.name + "Impl" + ".java", compileObjectInteraction(object,interaction,module.name));
			fsa.generateFile(pathName(module.name) + "/" + object.name+interaction.name + ".java",  GapeOutputConfigurationProvider::GEN_ONCE_OUTPUT , compileObjectInteractionAction(object,interaction,module.name));
		}
		
	}
		
	override compileObjectInteractionProperties(ObjectDeclaration object, InteractionDeclaration interaction, ImportManager im) '''
		«val manager = Utils.createJvmType("PersistenceManager","javax.jdo")»
		«super.compileObjectInteractionProperties(object,interaction,im)»
		private «im.serialize(manager)» persitenceManager; 
	'''
	override compileObjectInteractionMethods(ObjectDeclaration object, InteractionDeclaration interaction, ImportManager im) '''
		«val manager = Utils.createJvmType("PersistenceManager","javax.jdo")»
		«super.compileObjectInteractionMethods(object,interaction,im)»
		public void setPersitenceManager(«im.serialize(manager)» persitenceManager) {
				this.persitenceManager = persitenceManager;
		} 
		public «im.serialize(manager)»  getPersitenceManager() {
				return this.persitenceManager;
		} 
	'''

	/* Compile the domain class body */
	override compileDomainClassBody(ObjectDeclaration object, String targetPackage, ImportManager im) '''
		
		import javax.jdo.annotations.Persistent;
		import javax.jdo.annotations.PrimaryKey;
		import javax.jdo.annotations.PersistenceCapable;
		import javax.jdo.annotations.IdGeneratorStrategy;
		
		/**
		*	Class «object.name».
		*/ 
		@SuppressWarnings("all")
		@PersistenceCapable
		public class «object.name» extends «abstractClassName(object.name)» implements «interfaceName(object.name)» {
			«compilePrimaryKey(object,im)»
			«compileDomainClassProperties(object,targetPackage,im)»
			«compileDomainClassConstructor(object,im)»
			«compileDomainClassAccessors(object,im)»
			«compileDomainClassVirtualProperties(object,im)»
			«compileDomainClassMethods(object,im)»
			«compileConvenientCollectionMethods(object,im)»
		}
	'''
	
	/* Primary Key */
	def compilePrimaryKey(ObjectDeclaration declaration, ImportManager manager) '''
		/** Primary key. */
		@Persistent(valueStrategy = IdGeneratorStrategy.IDENTITY)
		@PrimaryKey
		private Long id;
	''' 
	
	/* Domain class properties */
	override compileDomainClassProperties(ObjectDeclaration object, String targetPackage, ImportManager im) '''
		«IF object.contract != null»
			«val contract = Utils.createJvmType(object.contract)»
			/* Contract «contractClassName(object.name)». */
			private «contractClassName(object.name)» contract;
		«ENDIF»
		«FOR p : object.properties»
			«IF Utils.isSupportedCollection( p.typeOrCollection.name)»
				«IF !"virtual".equalsIgnoreCase( p.virtual )»
					«propertyStorage(object,targetPackage,p)»
					protected «p.typeOrCollection.name»<«p.type.name»> «p.name»;
				«ENDIF»											
			«ELSE»
				«IF !"virtual".equalsIgnoreCase( p.virtual )»
					«propertyStorage(object,targetPackage,p)»
					protected «p.typeOrCollection.name» «p.name»;
				«ENDIF»
			«ENDIF»
		«ENDFOR»
	'''
	
	/* Objects declared as properties and collections are unowned */
	def propertyStorage(ObjectDeclaration object,String targetPackage,PropertyDeclaration property) '''
		/** «property.name » property. */
		«IF Utils.isSupportedCollection( property.typeOrCollection.name )»
		«log.debug("storage for: " + object.name + " -> " + property.name + " " + targetPackage + "." + property.typeOrCollection.name + " collection -> one to many ")»
		@com.google.appengine.datanucleus.annotations.Unowned
		@Persistent		
		«ELSEIF symbolTable.objects.containsKey( targetPackage + "." + property.typeOrCollection.name)»
			«log.debug("storage for: " + object.name + " -> "  + property.name + " " + targetPackage + "." + property.typeOrCollection.name  + " object -> one to one ")»			
			@com.google.appengine.datanucleus.annotations.Unowned
			@Persistent(defaultFetchGroup = "true")
		«ELSE»
			«log.debug("storage for: " + object.name + " -> " + property.name + " " + targetPackage + "." + property.typeOrCollection.name + " object supported")»
			@Persistent
		«ENDIF»		 
	'''
	
}