package com.mylaensys.dsl.generator.jdo

import com.mylaensys.dsl.generator.ServiceGenerator
import com.mylaensys.dsl.gape.ModuleDeclaration
import com.mylaensys.dsl.gape.ObjectDeclaration
import org.eclipse.xtext.generator.IFileSystemAccess
import com.mylaensys.dsl.generator.pojo.ServiceGeneratorPojo
import org.eclipse.xtext.xbase.compiler.ImportManager
import com.mylaensys.dsl.generator.SymbolTable
import com.mylaensys.dsl.generator.utils.Utils
import org.apache.log4j.Level
import com.mylaensys.dsl.gape.PropertyDeclaration
import com.mylaensys.dsl.gape.InteractionDeclaration

class ServiceGeneratorJDO extends ServiceGeneratorPojo implements ServiceGenerator {
	
	override compile(ModuleDeclaration module, ObjectDeclaration object, SymbolTable symbolTable ,IFileSystemAccess fsa) {
		log.setLevel( Level.DEBUG )
		/* Compile the Service */
		this.symbolTable = symbolTable
		fsa.generateFile(pathName(module.name) + "/" + serviceName(object.name) + ".java", compileServiceClass(object,module.name));		
	}
	/* Service Class Body */
	override compileServiceClassBody(ObjectDeclaration object, String targetPackage, ImportManager im) '''
		import javax.ws.rs.PathParam;
		import javax.ws.rs.QueryParam;
		import javax.ws.rs.core.Response;
		import javax.jdo.listener.LoadLifecycleListener;
		import javax.jdo.listener.StoreLifecycleListener;
		import javax.jdo.listener.CreateLifecycleListener;
		import javax.jdo.listener.DeleteLifecycleListener;

		/** «object.name». */
		«val name = serviceName(object.name)»	
		@javax.ws.rs.Path("/1.0/«Utils.makePlural(object.name.toLowerCase)»")
		public class «name» extends com.mylaensys.dsl.gape.AbstractService 
				implements 	LoadLifecycleListener, CreateLifecycleListener, 
							StoreLifecycleListener, DeleteLifecycleListener {
			«injectLogger(name,targetPackage,im)»

			«compileRegistry(object,im)»
			«compileMethodAll(object,targetPackage,im)»
			«compileMethodGet(object,targetPackage,im)»
			«compileMethodCreate(object,targetPackage,im)»
			«compileMethodUpdate(object,targetPackage,im)»
			«compileCollections(object,targetPackage,im)»
			«compileInteractions(object,targetPackage,im)»
			«compileCallbacks(object,targetPackage,im)»
			
		}
	'''
	
	
	/* Generates GET - /objects */
	override compileMethodAll(ObjectDeclaration object, String targetPackage, ImportManager im) '''
		«val clazz = Utils.createJvmType( object.name, targetPackage )»
		«val query = Utils.createJvmType("Query","javax.jdo")»
		«val manager = Utils.createJvmType("PersistenceManager","javax.jdo")»
		«val PMF = Utils.createJvmType("PMF","com.mylaensys.dsl.gape")»
		«val collection = Utils.createJvmType("List","java.util")»
		/**
		 * Returns all instances. 
		 * @param offset collection offset
		 * @param limit number of instances
		 * @return JSON collection of «im.serialize(clazz)»
		 */
		@javax.ws.rs.GET
		@javax.ws.rs.Path("/")
		@javax.ws.rs.Produces("application/json")
		public final Response «object.name.toLowerCase»All(@QueryParam("offset") final Long offset, @QueryParam("limit") final Long limit) {
			Response.Status status = Response.Status.BAD_REQUEST; 
			String message = "Bad Request";
			Long start = offset == null ? Long.valueOf(0L) : offset;
			Long end = limit == null ? Long.MAX_VALUE : start + limit;
			
			LOG.info("Received a READ ALL request for object «object.name» offset=" + start + " limit=" + end);
			
			«compileSecurity(object)»			
			
			«im.serialize(manager)» pm = «im.serialize(PMF)».get().getPersistenceManager();
			/* add the callback listener */
			pm.addInstanceLifecycleListener(this, null);
			try {
				«im.serialize(query)» query = pm.newQuery(«im.serialize(clazz)».class);
				/* Adjust range */
				query.setRange(start, end);						
				«im.serialize(collection)»<«im.serialize(clazz)»> all = («im.serialize(collection)»<«im.serialize(clazz)»>) query.execute(); 
				return Response.status(Response.Status.CREATED).entity(all).build();
			} catch (Exception e) {
				«compileException(object)»
			} finally {
				pm.close();
			}
			return Response.status(status).entity(message).build();
		}		
	'''
	
	
	/* Generates GET - /objects/{id} */
	override compileMethodGet(ObjectDeclaration object, String targetPackage, ImportManager im) '''
		«val clazz = Utils.createJvmType( object.name, targetPackage )»
		«val PMF = Utils.createJvmType("PMF","com.mylaensys.dsl.gape")»
		«val manager = Utils.createJvmType("PersistenceManager","javax.jdo")»
		/** 
		 * Returns a instance. 
		 * @param id key of the instance
		 * @return a «im.serialize(clazz)» instance 
		 */
		@javax.ws.rs.GET
		@javax.ws.rs.Path("/{id}")
		@javax.ws.rs.Produces("application/json")
		public final Response «object.name.toLowerCase»Read(@PathParam("id") final Long id) {
			Response.Status status = Response.Status.BAD_REQUEST;
			String message = "Bad Request";
			LOG.info("Received a READ request for object «object.name»(" + id + ")");
			
			«compileSecurity(object)»
			
			«im.serialize(manager)» pm = «im.serialize(PMF)».get().getPersistenceManager();
			/* add the callback listener */
			pm.addInstanceLifecycleListener(this, null);			
			try {
				«im.serialize(clazz)» object = pm.getObjectById(«im.serialize(clazz)».class, id);
				
				return Response.status(Response.Status.OK).entity(object).build();
			} catch (Exception e) {
				«compileException(object)»
			} finally {
				pm.close();
			}
			return Response.status(status).entity(message).build();			
		}
	'''
	/* Generates POST - /objects */
	override compileMethodCreate(ObjectDeclaration object, String targetPackage, ImportManager im) '''
		«val clazz = Utils.createJvmType( object.name, targetPackage )»
		«val response = Utils.createJvmType("SubmitResponse","com.mylaensys.dsl.gape")»
		«val hashmap = Utils.createJvmType("HashMap","java.util")»
		«val PMF = Utils.createJvmType("PMF","com.mylaensys.dsl.gape")»
		«val manager = Utils.createJvmType("PersistenceManager","javax.jdo")»
		/** 
		 * Creates a «im.serialize(clazz)» instance. 
		 * @param dto «im.serialize(clazz)» DTO
		 * @return a «im.serialize(clazz)» instance 
		 */
		@javax.ws.rs.POST
		@javax.ws.rs.Path("/")
		@javax.ws.rs.Consumes("application/json")
		@javax.ws.rs.Produces("application/json")
		public final Response «object.name.toLowerCase»Create(final «object.name + "DTO"» dto) {
			Response.Status status = Response.Status.BAD_REQUEST;
			String message = "Bad Request";
			LOG.info("Received a CREATE request for «object.name»");
			«compileSecurity(object)»
			
			«im.serialize(manager)» pm = «im.serialize(PMF)».get().getPersistenceManager();
			/* add the callback listener */
			pm.addInstanceLifecycleListener(this, null);			
			javax.jdo.Transaction txn = pm.currentTransaction();	
			try {
				txn.begin();
				«im.serialize(clazz)» toStore = new «im.serialize(clazz)»();
				«im.serialize(response)» response = submitForCreate(dto , toStore);
				
				«im.serialize(hashmap)»<String, String> propertyErrors = response.getPropertyErrors();
				if (propertyErrors.size() == 0) {
					«resolveOneToOneRelationship(object,targetPackage,"toStore",im)»
				
					pm.makePersistent(toStore);
					
					«/* Resolve cross dependencies */»
					«resolveOneToManyDependencies(object, targetPackage, "toStore",im)»					
					txn.commit();
						
					String url = javax.ws.rs.core.UriBuilder.fromUri("todo").path("/{id}").build(toStore.getId()).toString();
					java.net.URI location = new java.net.URI(url);
					return Response.created(location).status(Response.Status.CREATED).entity(toStore).build();
				} else {
					return Response.status(Response.Status.BAD_REQUEST).entity(propertyErrors).build();
				}
			} catch (Exception e) {
				«compileException(object)»
			} finally {
				if (txn.isActive()) {
					txn.rollback();
				}				
				pm.close();
			}
			return Response.status(status).entity(message).build();				
		}		
	'''
	/** Generates POST - /objects/{id} */
	override compileMethodUpdate(ObjectDeclaration object, String targetPackage, ImportManager im) '''		
		«val clazz = Utils.createJvmType( object.name, targetPackage )»
		«val audit = Utils.createJvmType("AuditData","com.mylaensys.dsl.gape")»
		«val response = Utils.createJvmType("SubmitResponse","com.mylaensys.dsl.gape")»
		«val hashmap = Utils.createJvmType("HashMap","java.util")»
		«val PMF = Utils.createJvmType("PMF","com.mylaensys.dsl.gape")»
		«val manager = Utils.createJvmType("PersistenceManager","javax.jdo")»
		/** 
		 * Updates a «im.serialize(clazz)» instance.
		 * @param id key of the instance
		 * @param dto «im.serialize(clazz)» DTO
		 * @return a «im.serialize(clazz)» instance 
		 */
		
		@javax.ws.rs.POST
		@javax.ws.rs.Path("/{id}")
		@javax.ws.rs.Consumes("application/json")
		@javax.ws.rs.Produces("application/json")
		public final Response «object.name.toLowerCase»Update(@PathParam("id") final Long id, final «object.name + "DTO"» dto) {		
			Response.Status status = Response.Status.BAD_REQUEST;
			String message = "Bad Request";
			LOG.info("Received an UPDATE request for object «object.name»(" + id + ")");
			
			«compileSecurity(object)»
			
			«im.serialize(manager)» pm = «im.serialize(PMF)».get().getPersistenceManager();
			/* add the callback listener */
			pm.addInstanceLifecycleListener(this, null);			
			javax.jdo.Transaction txn = pm.currentTransaction();	
			try {
				txn.begin();
				«im.serialize(clazz)» toUpdate = pm.getObjectById(«im.serialize(clazz)».class, id);
				«im.serialize(response)» response = submitForUpdate(dto , toUpdate);
				
				«im.serialize(hashmap)»<String, String> propertyErrors = response.getPropertyErrors();
				if (propertyErrors.size() == 0) {
					
					«resolveOneToOneRelationship(object,targetPackage,"toUpdate",im)»
				
					pm.makePersistent(toUpdate);
					
					«resolveOneToManyDependencies(object,targetPackage,"toUpdate",im)»
				
					txn.commit();
					return Response.status(Response.Status.OK).entity(toUpdate).build();
				} else {
					return Response.status(Response.Status.BAD_REQUEST).entity(propertyErrors).build();
				}
			} catch (Exception e) {
				«compileException(object)»
			} finally {
				if (txn.isActive()) {
					txn.rollback();
				}				
				pm.close();
			}
			return Response.status(status).entity(message).build();				
		}		
	'''
	
	/* Generates GET - /objects/{id}/collection */	
	override compileCollectionMethodGet(ObjectDeclaration object,String targetPackage, PropertyDeclaration property, ImportManager im)'''
		«log.debug("compiling collection method GET for object: " + object.name + " -> " + property.name)»
		
		«val clazz = Utils.createJvmType( object.name, targetPackage )»
		«val PMF = Utils.createJvmType("PMF","com.mylaensys.dsl.gape")»
		«val manager = Utils.createJvmType("PersistenceManager","javax.jdo")»		
		/**
		 * Returns all instances of the «property.name» collection. 
		 * @param offset collection offset
		 * @param limit number of instances
		 * @return JSON collection of «property.type.name»
		 */
		
		@javax.ws.rs.GET
		@javax.ws.rs.Path("/{id}/«property.name»")
		@javax.ws.rs.Produces("application/json")
		public final Response «property.name»Get(@PathParam("id") final Long id, @javax.ws.rs.QueryParam("offset") final Long offset, final @javax.ws.rs.QueryParam("limit") Long limit) {
			Response.Status status = Response.Status.BAD_REQUEST;
			String message = "Bad Request";
			LOG.info("Received a READ request for collection «property.name» object «object.name»");
			
			«compileSecurity(object)»
			
			«im.serialize(manager)» pm = «im.serialize(PMF)».get().getPersistenceManager();
			/* add the callback listener */
			pm.addInstanceLifecycleListener(this, null);
			try {
				«im.serialize(clazz)» «object.name.toLowerCase» = pm.getObjectById(«im.serialize(clazz)».class, id);				
				List<«property.type.name»> collection = new ArrayList<«property.type.name»>();
				for («property.type.name» o : «object.name.toLowerCase».get«property.name.toFirstUpper»()) {
					collection.add(o);
				} 

				return Response.status(Response.Status.OK).entity(collection).build();			
			} catch (Exception e) {
				«compileException(object)»
			} finally {
				pm.close();
			}
			return Response.status(status).entity(message).build();			
			
		}
	'''
	/* Generates POST - /objects/{id}/collection/{other} */
	override compileCollectionMethodAdd(ObjectDeclaration object, String targetPackage,PropertyDeclaration property, ImportManager im)'''
		«log.debug("compiling collection method ADD for object: " + object.name + " -> " + property.name)»
		«val clazz = Utils.createJvmType( object.name, targetPackage )»
		«val manager = Utils.createJvmType("PersistenceManager","javax.jdo")»	
		«val PMF = Utils.createJvmType("PMF","com.mylaensys.dsl.gape")»
		/**
		* Add a «property.type.name».
		* @param id key of the object
		* @param other key of the object to add
		* @return a JSON representation of the object
		*/
		@javax.ws.rs.POST
		@javax.ws.rs.Path("/{id}/«property.name»/{other}")
		@javax.ws.rs.Produces("application/json")
		public final Response «property.name»Add(@PathParam("id") final Long id, @PathParam("other") final Long other) {			
			Response.Status status = Response.Status.BAD_REQUEST;
			String message = "Bad Request";
			
			LOG.info("Received an ADD request for element (" + other + ") of collection «property.name» object «object.name»(" + id + ")");
			
			«compileSecurity(object)»
			
			«im.serialize(manager)» pm = «im.serialize(PMF)».get().getPersistenceManager();
			/* add the callback listener */
			pm.addInstanceLifecycleListener(this, null);			
			javax.jdo.Transaction txn = pm.currentTransaction();
			try {
				txn.begin();
								
				«im.serialize(clazz)» «object.name.toLowerCase» = pm.getObjectById(«im.serialize(clazz)».class, id);
				«property.type.name» toAdd = pm.getObjectById(«property.type.name».class, other);
				
				«object.name.toLowerCase».addTo«property.name.toFirstUpper»(toAdd);
				
				«resolveManyToManyDependencies(object,object.name.toLowerCase,targetPackage,"toAdd","addTo",im)»
				
				«property.typeOrCollection.name» collection = «object.name.toLowerCase».get«property.name.toFirstUpper»();
				
				txn.commit();
				return Response.status(Response.Status.OK).entity(collection).build();
			} catch (RuntimeException e) {
					return Response.status(Response.Status.BAD_REQUEST).entity(e.getMessage()).build();
			} catch (Exception e) {
				«compileException(object)»
			} finally {
				if (txn.isActive()) {
					txn.rollback();
				}				
				pm.close();
			}
			return Response.status(status).entity(message).build();			
			
		}
	'''
	
	/* Generates DELETE  /objects/{id}/collection/{other} */
	override compileCollectionMethodRemove(ObjectDeclaration object,String targetPackage, PropertyDeclaration property, ImportManager im)'''
		«val clazz = Utils.createJvmType( object.name, targetPackage )»
		«val manager = Utils.createJvmType("PersistenceManager","javax.jdo")»
		«val PMF = Utils.createJvmType("PMF","com.mylaensys.dsl.gape")»
	
		«log.debug("compiling collection method REMOVE for object: " + object.name + " -> " + property.name)»
		/**
		 * Delete an object. 
		 * @param id key of object
		 * @param other not specified
		 * @return not specified
		 */
		@javax.ws.rs.DELETE
		@javax.ws.rs.Path("/{id}/«property.name»/{other}")
		@javax.ws.rs.Produces("application/json")
		public final Response «property.name»Remove(@PathParam("id") final Long id, @PathParam("other") final Long other) {			
			Response.Status status = Response.Status.BAD_REQUEST;
			String message = "Bad Request";
			
			LOG.info("Received an DELETE request for element (" + other + ") of collection «property.name» object «object.name»(" + id + ")");
			
			«compileSecurity(object)»
			
			«im.serialize(manager)» pm = «im.serialize(PMF)».get().getPersistenceManager();
			/* add the callback listener */
			pm.addInstanceLifecycleListener(this, null);			
			javax.jdo.Transaction txn = pm.currentTransaction();
			try {
				txn.begin();
								
				«im.serialize(clazz)» «object.name.toLowerCase» = pm.getObjectById(«im.serialize(clazz)».class, id);
				«property.type.name» toRemove = pm.getObjectById(«property.type.name».class, other);
				
				«object.name.toLowerCase».removeFrom«property.name.toFirstUpper»(toRemove);
				
				«resolveManyToManyDependencies(object,object.name.toLowerCase,targetPackage,"toRemove","removeFrom",im)» 
				
				«property.typeOrCollection.name» collection = «object.name.toLowerCase».get«property.name.toFirstUpper»();
				
				txn.commit();
				return Response.status(Response.Status.OK).entity(collection).build();
			} catch (Exception e) {
				«compileException(object)»
			} finally {
				if (txn.isActive()) {
					txn.rollback();
				}				
				pm.close();
			}
			return Response.status(status).entity(message).build();			
			
		}		
	''' 
	/* Creates and add to the collection */
	override compileCollectionMethodCreate(ObjectDeclaration object, String targetPackage, PropertyDeclaration property, ImportManager im) '''		
		«val clazz = Utils.createJvmType( object.name, targetPackage )»
		«val PMF = Utils.createJvmType("PMF","com.mylaensys.dsl.gape")»		
		«val manager = Utils.createJvmType("PersistenceManager","javax.jdo")»		
		«val audit = Utils.createJvmType("AuditData","com.mylaensys.dsl.gape")»
		«val response = Utils.createJvmType("SubmitResponse","com.mylaensys.dsl.gape")»
		«val hashmap = Utils.createJvmType("HashMap","java.util")»
		«val toAddType = symbolTable.objects.get( targetPackage + "." + property.type.name )»
		
		«log.debug("compiling collection method CREATE and ADD for object: " + object.name + " -> " + property.name)»
		/**
		 * Creates and add a «property.type.name». 
		 * @param id key of the object
		 * @param dto DTO 
		 * @return an JSON instance of the added object
		 */
		@javax.ws.rs.POST
		@javax.ws.rs.Path("/{id}/«property.name»")
		@javax.ws.rs.Consumes("application/json")
		@javax.ws.rs.Produces("application/json")
		public final Response  «property.name»CreateAndAdd(@PathParam("id") final Long id, final «property.type.name + "DTO"» dto) {			
			Response.Status status = Response.Status.BAD_REQUEST;
			String message = "Bad Request";
			
			«compileSecurity(object)»
			
			«im.serialize(manager)» pm = «im.serialize(PMF)».get().getPersistenceManager();
			/* add the callback listener */
			pm.addInstanceLifecycleListener(this, null);
			javax.jdo.Transaction txn = pm.currentTransaction();
			try {
				txn.begin();
				«im.serialize(clazz)» «object.name.toLowerCase»0 = pm.getObjectById(«im.serialize(clazz)».class, id);
				«property.type.name» toAdd = new «property.type.name»();
				
				«im.serialize(response)» response = submitForCreate(dto, toAdd);
				
				«im.serialize(hashmap)»<String, String> propertyErrors = response.getPropertyErrors();
				if (propertyErrors.size() == 0) {
					«resolveOneToOneRelationship(toAddType,targetPackage,"toAdd",im)»
				
					pm.makePersistent(toAdd);
					
					«/* Resolve cross dependencies */»
					«resolveOneToManyDependencies(toAddType,targetPackage,"toAdd",im)»
					
					«object.name.toLowerCase»0.addTo«property.name.toFirstUpper»(toAdd);
										
					«resolveManyToManyDependencies(object,object.name.toLowerCase+"0",targetPackage,"toAdd","addTo",im)»
					
					pm.makePersistent(«object.name.toLowerCase»0);
					
					txn.commit();
					
					String url = javax.ws.rs.core.UriBuilder.fromUri("todo").path("/{id}").build(toAdd.getId()).toString();
					java.net.URI location = new java.net.URI(url);
					return Response.created(location).status(Response.Status.CREATED).entity(toAdd).build();
					
				} else {
					return Response.status(Response.Status.BAD_REQUEST).entity(propertyErrors).build();
				}
			} catch (Exception e) {
				«compileException(object)»
			} finally {
				if (txn.isActive()) {
					txn.rollback();
				}				
				pm.close();
			}
			return Response.status(status).entity(message).build();			
			
		}		
	'''	
	
	/* Resolve one-to-many dependencies (<->) operator */
	def resolveOneToManyDependencies(ObjectDeclaration object,String targetPackage, String targetObject, ImportManager im) '''
		«log.debug("Resolving one-to-many dependencies for object: " + object.name)»
		«FOR p : object.properties»
			«IF !Utils.isSupportedCollection(p.typeOrCollection.name )»
				«IF symbolTable.objects.containsKey( targetPackage + "." + p.typeOrCollection.name )»
					«IF p.targetRef != null»
					«log.debug("Resolving one-to-many dependencies for object: " + object.name + " -> " + p.name + " has a one-to-many dependency to " + p.targetRef.name + ":" + p.ref )»
					/* Detects if the «p.name» is changed */	
					Long current«p.name.toFirstUpper» = (Long) change«p.name.toFirstUpper».getCurrentValue();
					Long previous«p.name.toFirstUpper» = (Long) change«p.name.toFirstUpper».getPreviousValue();
					
					if (current«p.name.toFirstUpper».compareTo(previous«p.name.toFirstUpper» == null ? Long.valueOf(-1L) : previous«p.name.toFirstUpper») != 0) {
						LOG.info("Resolving one-to-many dependencies for «object.name».«p.name», previous«p.name.toFirstUpper»=" + previous«p.name.toFirstUpper» + ",current«p.name.toFirstUpper»=" + current«p.name.toFirstUpper»);
						if (previous«p.name.toFirstUpper» != null) {
							«p.typeOrCollection.name» p«p.name» = pm.getObjectById(«p.typeOrCollection.name».class, previous«p.name.toFirstUpper»);
							p«p.name».removeFrom«p.ref.toFirstUpper»(«targetObject»); 
						} 
						«p.typeOrCollection.name» c«p.name» = «targetObject».get«p.name.toFirstUpper»(); 
						c«p.name».addTo«p.ref.toFirstUpper»(«targetObject»);
						pm.makePersistent(c«p.name»); 
					}									
					«ENDIF»
				«ELSE»
				«ENDIF»
			«ENDIF»
		«ENDFOR»
	'''
	/* Resolve many-to-many dependencies (<->) operator */
	def resolveManyToManyDependencies(ObjectDeclaration object, String instance,String targetPackage,String targetObject, String action, ImportManager im) '''
		«log.debug("Resolving many-to-many dependencies for object: " + object.name)»
		«FOR p : object.properties»
			«IF Utils.isSupportedCollection( p.typeOrCollection.name )»
				«IF p.targetRef != null»
					«IF p.targetRef.name.equalsIgnoreCase( p.type.name )»
						«log.debug("Resolving many-to-many dependencies for object: " + object.name + " -> " + p.name + " has a many-to-many dependency to " + p.targetRef.name + ":" + p.ref )»
						«/* Detect id the ref is a collection */»
						«IF Utils.isSupportedCollection( symbolTable.getProperty( targetPackage + "." + p.targetRef.name , p.ref  ).typeOrCollection.name) »
							«targetObject».«action»«p.ref.toFirstUpper»(«instance»);
							pm.makePersistent(«targetObject»);
						«ENDIF»
					«ENDIF»
				«ENDIF»
			«ENDIF»
		«ENDFOR»
	'''

	/* Resolve OneToOne relationships */
	def resolveOneToOneRelationship(ObjectDeclaration object,String targetPackage, String targetObject, ImportManager im)'''
		«val audit = Utils.createJvmType("AuditData","com.mylaensys.dsl.gape")»
		«log.debug("Resolving relationships for object: " + object.name )»
		«FOR p : object.properties»
			«IF !"virtual".equalsIgnoreCase( p.virtual )»
				«IF !Utils.isSupportedCollection( p.typeOrCollection.name )»
					«log.debug("Resolving relationships for object: " + object.name + " -> " + p.name)»
					«IF symbolTable.objects.containsKey( targetPackage + "." + p.typeOrCollection.name)»
						«log.debug("Resolving relationships for object: " + object.name + " -> " + p.name + " is a dependent object ")»
						/* Resolve «p.name» dependency */
						«im.serialize(audit)» change«p.name.toFirstUpper»  = response.getChanges().get("«p.name»");
						LOG.info("Resolving relationships for «object.name».«p.name», previous«p.name.toFirstUpper»=" + change«p.name.toFirstUpper».getPreviousValue() + ",current«p.name.toFirstUpper»=" + change«p.name.toFirstUpper».getCurrentValue());
						try {
							if (change«p.name.toFirstUpper».getCurrentValue() != null) {
								«p.typeOrCollection.name» «p.name» = pm.getObjectById(«p.typeOrCollection.name».class, change«p.name.toFirstUpper».getCurrentValue());
								«targetObject».set«p.name.toFirstUpper»(«p.name»);
							} else {
								
							}
						} catch (javax.jdo.JDOObjectNotFoundException e) {
							propertyErrors.put("«p.name»", "Invalid «p.name»(" + change«p.name.toFirstUpper».getCurrentValue() + ")");
							return Response.status(Response.Status.BAD_REQUEST).entity(propertyErrors).build();
						}
					«ENDIF»
				«ENDIF»
			«ENDIF»	
		«ENDFOR»
	'''
	/* Compile JDO callback */
	def compileCallbacks(ObjectDeclaration declaration, String string, ImportManager im) '''
	«val listener = Utils.createJvmType("InstanceLifecycleEvent","javax.jdo.listener")»	
	@Override
	public void postLoad(final «im.serialize(listener)» instanceLifecycleEvent) { }
	@Override
	public void postCreate(final «im.serialize(listener)»  instanceLifecycleEvent) { }
	@Override
	public void preStore(final «im.serialize(listener)»  instanceLifecycleEvent) { }	
	@Override
	public void postStore(final «im.serialize(listener)»  instanceLifecycleEvent) { }
	@Override
	public void preDelete(final «im.serialize(listener)»  instanceLifecycleEvent) { }	
	@Override
	public void postDelete(final «im.serialize(listener)»  instanceLifecycleEvent) { }	
	'''
	/* Compile the interactions */
	def compileInteractions(ObjectDeclaration object, String targetPackage, ImportManager im) '''
		«FOR i : object.interactions»
			«complileInteractionMethod(object,i,targetPackage,im)»
		«ENDFOR»
	'''
	/* Compile the interactions method */
	def complileInteractionMethod(ObjectDeclaration object, InteractionDeclaration interaction, String targetPackage, ImportManager im) '''
		«val clazz = Utils.createJvmType( object.name, targetPackage )»
		«val PMF = Utils.createJvmType("PMF","com.mylaensys.dsl.gape")»
		«val manager = Utils.createJvmType("PersistenceManager","javax.jdo")»
		@javax.ws.rs.POST
		@javax.ws.rs.Path("/{id}/«interaction.name.toLowerCase»")
		@javax.ws.rs.Consumes("application/json")
		@javax.ws.rs.Produces("application/json")
		public Response «interaction.name.toLowerCase»(@PathParam("id") Long id) {
			Response.Status status = Response.Status.BAD_REQUEST;
			String message = "Bad Request";
			LOG.info("Received a «interaction.name» request for object «object.name»(" + id + ")");
			
			«compileSecurity(object)»
			
			«im.serialize(manager)» pm = «im.serialize(PMF)».get().getPersistenceManager();
			/* add the callback listener */
			pm.addInstanceLifecycleListener(this, null);			
			try {
				«im.serialize(clazz)» object = pm.getObjectById(«im.serialize(clazz)».class, id);
				«object.name»«interaction.name» interaction = new «object.name»«interaction.name»();
				interaction.setPersitenceManager(pm);
				interaction.setSelf(object);
				java.lang.Object response = interaction.run(); 
				
				return Response.status(Response.Status.OK).entity(response).build();
			} catch (Exception e) {
				«compileException(object)»
			} finally {
				pm.close();
			}
			return Response.status(status).entity(message).build();			
		}
	'''

}