package com.mylaensys.dsl.generator.pojo

import com.mylaensys.dsl.generator.ModelGenerator
import com.mylaensys.dsl.gape.ObjectDeclaration
import org.eclipse.xtext.generator.IFileSystemAccess
import org.eclipse.xtext.common.types.TypesFactory
import org.eclipse.xtext.xbase.compiler.ImportManager
import com.mylaensys.dsl.gape.ModuleDeclaration
import com.mylaensys.dsl.gape.PropertyDeclaration
import org.eclipse.emf.common.util.EList
import com.mylaensys.dsl.generator.SymbolTable
import org.apache.log4j.Logger
import com.mylaensys.dsl.gape.Permission
import org.eclipse.xtext.common.types.JvmType
import com.mylaensys.dsl.generator.utils.Utils
import com.mylaensys.dsl.GapeOutputConfigurationProvider
import java.util.ArrayList
import com.mylaensys.dsl.gape.InteractionDeclaration
import com.mylaensys.dsl.gape.MethodDeclaration

class ModelGeneratorPojo implements ModelGenerator {
	protected static Logger log =  Logger.getLogger("ModelGenerator")
	private int index = 0;	
	protected SymbolTable symbolTable
	
	override compile(ModuleDeclaration module,ObjectDeclaration object, SymbolTable symbolTable,IFileSystemAccess fsa) {
		
		/* Compile the Interface */
		this.symbolTable = symbolTable
		fsa.generateFile(pathName(module.name) + "/" + interfaceName(object.name) + ".java", compileInterface(object,module.name));
		/* Compile the Abstract Class */
		fsa.generateFile(pathName(module.name) + "/" + abstractClassName(object.name) + ".java", /*GapeOutputConfigurationProvider::GEN_ONCE_OUTPUT ,*/ compileAbstractClass(object,module.name));
		/* Compile the Domain Class */
		fsa.generateFile(pathName(module.name) + "/" + object.name + ".java", compileDomainClass(object,module.name));
		/* Compile the Data Transfer object */
		fsa.generateFile(pathName(module.name) + "/" + dtoClassName(object.name) + ".java", compileDataTransferObjectClass(object,module.name));
		/* Compile the Contract Interface */
		fsa.generateFile(pathName(module.name) + "/" + contractClassName(object.name) + ".java", compileContractInterface(object,module.name));
		/* Compile the Interaction Interfaces */		
		for(interaction : symbolTable.interactionInterfaces.keySet) {
			fsa.generateFile(pathName(module.name) + "/" + interaction + ".java", compileInteractionInterface(object,interaction,module.name));
		}
		
		for(interaction : object.interactions ) {
			fsa.generateFile(pathName(module.name) + "/" + object.name+interaction.name + "Impl"+ ".java", compileObjectInteraction(object,interaction,module.name));
			fsa.generateFile(pathName(module.name) + "/" + object.name+interaction.name + ".java", compileObjectInteractionAction(object,interaction,module.name));
		}
	}
	
	def compileContractInterface(ObjectDeclaration object, String targetPackage) '''
		«log.debug("compiling: " + contractClassName( object.name ) )»
		«val importManager = new ImportManager(true,Utils.createJvmType( contractClassName( object.name ) , targetPackage ))»
		«val body = compileContractInterfaceBody(object,importManager)»
		package «targetPackage»;
		import java.util.*;
		import java.math.*;
		«FOR i : importManager.imports»
			import «i»;
		«ENDFOR»
		«body»
	'''
	
	def compileContractInterfaceBody(ObjectDeclaration object, ImportManager im) '''
		«val round = Utils.createJvmType("Round","com.mylaensys.dsl.gape")»
		/** Contract interface. */
		public interface «contractClassName( object.name )» {
			/* Entry and exit of normal public methods; exit of constructor. */
			/** invariant. */ 
			void invariant();
			«FOR p : object.properties»
				«IF Utils.isSupportedCollection( p.typeOrCollection.name )»
					«IF !"virtual".equalsIgnoreCase( p.virtual )»
						«IF Utils.isPermissionGranted( p.permissions, "write" )»
							/**
							 * Add «p.type.name» contract.
							 * @param round pre or post
							 * @param toAdd «p.type.name» to add 
							 */
							void addTo«p.name.toFirstUpper»(«im.serialize(round)» round, «p.type.name» toAdd);
						«ENDIF»
						«IF Utils.isPermissionGranted( p.permissions, "delete" )»
							/**
							 * Remove a «p.type.name» contract.
							 * @param round pre or post
							 * @param toRemove «p.type.name» to remove 
							 */
							void removeFrom«p.name.toFirstUpper»(«im.serialize(round)» round, «p.type.name» toRemove);
						«ENDIF»
					«ELSE»
						«/* Virtual properties has no parameters */»
						/**
						 * Virtual collection «p.name» contract. 
						 * @param round pre or post
						 */
						void «p.name.toLowerCase»(«im.serialize(round)» round);					
					«ENDIF»
				«ELSE»
					«IF !"virtual".equalsIgnoreCase( p.virtual )»
						/** 
						* «p.name» contract.
						* @param round pre or post
						* @param «p.name» property
						*/
						void «p.name.toLowerCase»(«im.serialize(round)» round, «p.typeOrCollection.name» «p.name»);
					«ELSE»
						«/* Virtual properties has no parameters */»
						/**
						 * Virtual property «p.name» contract. 
						 * @param round pre or post
						 */
						void «p.name.toLowerCase»(«im.serialize(round)» round);
					«ENDIF»					
				«ENDIF»	
			«ENDFOR»
			«FOR m : object.methods»
			/* Method  «m.name» */
			public void «m.name»(«im.serialize(round)» round,«FOR p : m.parameters SEPARATOR ','»«p.type.name» «p.name»«ENDFOR»);
		«ENDFOR»
			
		}
	'''
	
	def compileObjectInteractionAction(ObjectDeclaration object, InteractionDeclaration interaction, String targetPackage) '''
		«log.debug("compiling: " + object.name + interaction.name )»
		«val importManager = new ImportManager(true,Utils.createJvmType( object.name + interaction.name , targetPackage ))»
		«val body = compileObjectInteractionActionBody(object,interaction,importManager)»
		package «targetPackage»;
		import java.util.*;
		import java.math.*;
		«FOR i : importManager.imports»
			import «i»;
		«ENDFOR»
		«body»
	'''
	
	def compileObjectInteractionActionBody(ObjectDeclaration object, InteractionDeclaration interaction, ImportManager im) '''
		/* Class «object.name + interaction.name » */
		@SuppressWarnings("all")
		public class «object.name + interaction.name» extends «object.name + interaction.name»Impl implements com.mylaensys.dsl.gape.Interaction {
			public Object run() {
				throw new UnsupportedOperationException("TODO: provide implementation"); 
			}
		}
	'''
	
	def compileObjectInteraction(ObjectDeclaration object, InteractionDeclaration interaction, String targetPackage) '''
		«log.debug("compiling: " + object.name + interaction.name + "Impl")»
		«val importManager = new ImportManager(true,Utils.createJvmType( object.name + interaction.name , targetPackage ))»
		«val body = compileObjectInteractionBody(object,interaction,importManager)»
		package «targetPackage»;
		import java.util.*;
		import java.math.*;
		«FOR i : importManager.imports»
			import «i»;
		«ENDFOR»
		«body»
	'''
	
	def compileObjectInteractionBody(ObjectDeclaration object, InteractionDeclaration interaction, ImportManager im) '''
		/* Class «object.name + interaction.name » */
		@SuppressWarnings("all")
		public class «object.name + interaction.name»Impl {
			«compileObjectInteractionProperties(object,interaction,im)»
			«compileObjectInteractionMethods(object,interaction,im)»
		}
	'''
	def compileObjectInteractionProperties(ObjectDeclaration object, InteractionDeclaration interaction, ImportManager im) '''
		«IF symbolTable.interactionInterfaces.containsKey(object.name + "$" + interaction.name)»
			protected «object.name + "$" + interaction.name» self; 
		«ELSE»
			protected «object.name» self;
		«ENDIF»
	'''
	def compileObjectInteractionMethods(ObjectDeclaration object, InteractionDeclaration interaction, ImportManager im) '''
		«IF symbolTable.interactionInterfaces.containsKey(object.name + "$" + interaction.name)»
			public void setSelf(«object.name + "$" + interaction.name» self) {
				this.self = self;
			} 
		«ELSE»
			public void setSelf(«object.name» self) {
				this.self = self;
			} 
		«ENDIF»	
	'''
	
	def compileInteractionInterface(ObjectDeclaration object,String name,String targetPackage) '''
		«log.debug("compiling: " + name )»
		«val importManager = new ImportManager(true,Utils.createJvmType( name, targetPackage ))»
		«val body = compileInteractionInterfaceBody(object,name,importManager)»
		package «targetPackage»;
		import java.util.*;
		import java.math.*;
		«FOR i : importManager.imports»
			import «i»;
		«ENDFOR»
		«body»
	'''
	
	def compileInteractionInterfaceBody(ObjectDeclaration object,String name, ImportManager im) '''
		«val set = symbolTable.interactionInterfaces.get(name)»
		public interface «name» extends «interfaceName(object.name)» {
			«FOR m : set»
				/* Method  «m.name» */
				public «m.returnType.name» «m.name»(«FOR p : m.parameters SEPARATOR ','»«p.type.name» «p.name»«ENDFOR»);		
			«ENDFOR»
		}
	'''
	
	
	/* Compile the Data Transfer Object  */
	def compileDataTransferObjectClass(ObjectDeclaration object, String targetPackage)'''
		«log.debug("compiling: " + dtoClassName(object.name) )»
		«val importManager = new ImportManager(true,Utils.createJvmType( object.name, targetPackage ))»
		«val body = compileDataTransferObjectClassBody(object,targetPackage,importManager)»
		package «targetPackage»;
		import java.util.*;
		import java.math.*;
		«FOR i : importManager.imports»
			import «i»;
		«ENDFOR»
		«body»
	'''
	/* Compile the Data Transfer Object class Body */
	def compileDataTransferObjectClassBody(ObjectDeclaration object,String targetPackage, ImportManager im)'''
		«val name = dtoClassName(object.name)»
		/** Class «name». */
		@SuppressWarnings("all")
		public class «name» {
			«compileDataTransferObjectClassProperties(object,targetPackage,im)»
			«compileDataTransferObjectClassAccessor(object,targetPackage,im)»
		}
	'''
	/*  Compile the Data Transfer Object accessors */
	def compileDataTransferObjectClassAccessor(ObjectDeclaration object, String targetPackage,ImportManager im) '''
		«var writables = getWriteableProperties(object.properties)»
		«FOR p : writables»
			«IF symbolTable.objects.containsKey(targetPackage + "." + p.typeOrCollection.name)»
				/** Returns the «p.name» reference. @return «p.name» */
				public Long get«p.name.toFirstUpper»() {
					return this.«p.name»;
				}
				/** Set the «p.name» reference. @param param «p.name» */
				public final void set«p.name.toFirstUpper»(final Long param) {
					this.«p.name» = param;
				}
			«ELSE»
				/** Returns the «p.name» property. @return «p.name» */
				public final «p.typeOrCollection.name» get«p.name.toFirstUpper»() {
					return this.«p.name»;
				}
				/** Set the «p.name» property. @param param «p.name» */
				public final void set«p.name.toFirstUpper»(final «p.typeOrCollection.name» param) {
					this.«p.name» = param;
				}
			«ENDIF»
		«ENDFOR»
	'''
	
	/* Compile the Data Transfer Object class Properties */
	def compileDataTransferObjectClassProperties(ObjectDeclaration object,String targetPackage, ImportManager im) '''
		«var writables = getWriteableProperties(object.properties)»
		«FOR p : writables»
			«IF symbolTable.objects.containsKey( targetPackage +"." + p.typeOrCollection.name)»
				/** «p.name » reference. */
				private Long «p.name»;			
			«ELSE»
				/** «p.name » property. */
				private «p.typeOrCollection.name» «p.name»;
			«ENDIF»
		«ENDFOR»
	'''
	
	/* Compile the domain class  */
	def compileDomainClass(ObjectDeclaration object, String targetPackage ) '''
		«log.debug("compiling: " + object.name )»
		«val importManager = new ImportManager(true,Utils.createJvmType( object.name, targetPackage ))»
		«val body = compileDomainClassBody(object,targetPackage,importManager)»
		package «targetPackage»;
		
		import java.util.*;
		import java.math.*;
		
		«FOR i : importManager.imports»
			import «i»;
		«ENDFOR»
		«body»
	'''
	/* Compile the domain class body */
	def compileDomainClassBody(ObjectDeclaration object, String targetPackage, ImportManager im) '''
		/**
		* Class «object.name». 
		*/
		@SuppressWarnings("all")
		public class «object.name»  extends «abstractClassName(object.name)» implements «interfaceName(object.name)» {
			«compileDomainClassConstructor(object,im)»
			«compileDomainClassProperties(object,targetPackage,im)»
			«compileDomainClassAccessors(object,im)»
			«compileDomainClassVirtualProperties(object,im)»
			«compileDomainClassMethods(object,im)»
			«compileConvenientCollectionMethods(object,im)»
		}
	'''
	def compileDomainClassMethods(ObjectDeclaration object, ImportManager im) '''
		«FOR m : object.methods»
			/* Method  «m.name» */
			«IF !m.interactions.empty»
			@Override
			«ENDIF»
			public «m.returnType.name» «m.name»(«FOR p : m.parameters SEPARATOR ','»«p.type.name» «p.name»«ENDFOR») {
				«compileInvariants(object)»
				«compilePreConditionsMethod(object,m,im)»
				«m.returnType.name» r = super.«m.name»(«FOR p : m.parameters SEPARATOR ','»«p.name»«ENDFOR»);
				«compileInvariants(object)»
				«compilePostConditionsMethod(object,m,im)»
				return r;
			}
		«ENDFOR»
	'''
	
	/* Compile the domain class constructor */
	def compileDomainClassConstructor(ObjectDeclaration object, ImportManager im) '''
		/** Default Constructor. */
		public «object.name»() {
			this.self =  this;
			super.initialize(this);
			«IF object.contract != null»
			«val contract = Utils.createJvmType(object.contract)»
			this.contract = new «im.serialize(contract)»(this); 
			«compileInvariants(object)»
			«ENDIF»
		}
	'''
	
	/* Compile the domain class accessors */
	def compileDomainClassAccessors(ObjectDeclaration object, ImportManager im) '''
		/** Returns id property. @return id */
		@org.codehaus.jackson.annotate.JsonProperty
		public final Long getId() {
			return this.id;
		}
		/** Set the id property. @param param id */
		@org.codehaus.jackson.annotate.JsonProperty
		public final void setId(final Long param) {
			this.id = param;
		}
		«FOR p : object.properties»			
			«IF Utils.isSupportedCollection( p.typeOrCollection.name )»
				«IF !"virtual".equalsIgnoreCase( p.virtual )»
					«/* Collections are never serialized */»
					/** Returns the «p.name» collection. @return «p.name» */
					@org.codehaus.jackson.annotate.JsonIgnore
					public final «p.typeOrCollection.name»<«p.type.name»> get«p.name.toFirstUpper»() {
						return this.«p.name»;
					}
					/** Sets the «p.name» collection. @param collection «p.typeOrCollection.name» of «p.type.name» */
					public final void set«p.name.toFirstUpper»(final «p.typeOrCollection.name»<«p.type.name»> collection) {
						this.«p.name» = collection;
					}
				«ENDIF»
			«ELSE»
				«IF !"virtual".equalsIgnoreCase( p.virtual )»
					/** Returns the «p.name» property. @return «p.name» */
					«compilePropertyAllow(object,p,"read")» 
					public final «p.typeOrCollection.name» get«p.name.toFirstUpper»() {
						«compileInvariants(object)»
						«compilePostConditions(object,p,im)»
						return «p.name»;
					}
					/** Set the «p.name» property. @param param «p.name» */
					«compilePropertyAllow(object,p,"write")»
					public final void set«p.name.toFirstUpper»(final «p.typeOrCollection.name» param) {
						«compileInvariants(object)»
						«compilePreConditions(object,p,im)»
						this.«p.name» = param;
						«compileInvariants(object)»						
						«compilePostConditions(object,p,im)»
					}
				«ENDIF» 
			«ENDIF»
		«ENDFOR»
	'''
	def compileInvariants(ObjectDeclaration object)'''
		«IF object.contract != null»
		this.contract.invariant();
		«ENDIF»
	'''
	def compilePreConditions(ObjectDeclaration object, PropertyDeclaration property,ImportManager im) '''
		«val round = Utils.createJvmType("Round","com.mylaensys.dsl.gape")»
		«IF object.contract != null»
		this.contract.«property.name.toLowerCase()»(«im.serialize(round)».PreConditions,param);
		«ENDIF»
	'''
	def compilePreConditionsAddTo(ObjectDeclaration object, PropertyDeclaration property,ImportManager im) '''
		«val round = Utils.createJvmType("Round","com.mylaensys.dsl.gape")»
		«IF object.contract != null»
		this.contract.addTo«property.name.toFirstUpper»(«im.serialize(round)».PreConditions,toAdd);
		«ENDIF»
	'''
	def compilePostConditionsAddTo(ObjectDeclaration object, PropertyDeclaration property,ImportManager im) '''
		«val round = Utils.createJvmType("Round","com.mylaensys.dsl.gape")»
		«IF object.contract != null»
		this.contract.addTo«property.name.toFirstUpper»(«im.serialize(round)».PostConditions,toAdd);
		«ENDIF» 
	'''
	
	def compilePreConditionsRemoveFrom(ObjectDeclaration object, PropertyDeclaration property,ImportManager im) '''
		«val round = Utils.createJvmType("Round","com.mylaensys.dsl.gape")»
		«IF object.contract != null»
		this.contract.removeFrom«property.name.toFirstUpper»(«im.serialize(round)».PreConditions,toRemove);
		«ENDIF»
	'''
	def compilePostConditionsRemoveFrom(ObjectDeclaration object, PropertyDeclaration property,ImportManager im) '''
		«val round = Utils.createJvmType("Round","com.mylaensys.dsl.gape")»
		«IF object.contract != null»
		this.contract.removeFrom«property.name.toFirstUpper»(«im.serialize(round)».PostConditions,toRemove);
		«ENDIF»
	'''
	
	def compilePreConditionsMethod(ObjectDeclaration object, MethodDeclaration method,ImportManager im) '''
		«val round = Utils.createJvmType("Round","com.mylaensys.dsl.gape")»
		«IF object.contract != null»
		this.contract.«method.name»(«im.serialize(round)».PreConditions,«FOR p : method.parameters SEPARATOR ','»«p.name»«ENDFOR»);
		«ENDIF»
	'''
	
	def compilePostConditionsMethod(ObjectDeclaration object, MethodDeclaration method,ImportManager im) '''
		«val round = Utils.createJvmType("Round","com.mylaensys.dsl.gape")»
		«IF object.contract != null»
		this.contract.«method.name»(«im.serialize(round)».PostConditions,«FOR p : method.parameters SEPARATOR ','»«p.name»«ENDFOR»);
		«ENDIF»
	'''
	
	
	def compilePostConditions(ObjectDeclaration object, PropertyDeclaration property,ImportManager im) '''
		«val round = Utils.createJvmType("Round","com.mylaensys.dsl.gape")»
		«IF object.contract != null»
		this.contract.«property.name.toLowerCase()»(«im.serialize(round)».PostConditions,«property.name»);
		«ENDIF»
	'''
	
	def compilePostConditionsVirtual(ObjectDeclaration object, PropertyDeclaration property,ImportManager im) '''
		«val round = Utils.createJvmType("Round","com.mylaensys.dsl.gape")»
		«IF object.contract != null»
		this.contract.«property.name.toLowerCase()»(«im.serialize(round)».PostConditions);
		«ENDIF»
	'''
	
	
	/* Compile the domain class properties */
	def compileDomainClassProperties(ObjectDeclaration object, String targetPackage, ImportManager im) '''
		«IF object.contract != null»
			«val contract = Utils.createJvmType(object.contract)»
			private «contractClassName(object.name)» contract;
		«ENDIF»
		«FOR p : object.properties»
			«IF Utils.isSupportedCollection( p.typeOrCollection.name)»
				«IF !"virtual".equalsIgnoreCase( p.virtual )»
					protected «p.typeOrCollection.name»<«p.type.name»> «p.name»;
				«ENDIF»											
			«ELSE»
				«IF !"virtual".equalsIgnoreCase( p.virtual )»
					protected «p.typeOrCollection.name» «p.name»;
				«ENDIF»
			«ENDIF»
		«ENDFOR»
	'''
	
	
	/* Compile the abstract class */
	def compileAbstractClass(ObjectDeclaration object, String targetPackage) '''
		«log.debug("compiling: " + abstractClassName(object.name) )»
		«val name = abstractClassName(object.name)»
		«val importManager = new ImportManager(true,Utils.createJvmType( name, targetPackage ))»
		
		«val body = compileAbstractClassBody(object,importManager)»
		package «targetPackage»;

		import java.util.*;
		import java.math.*;
		«FOR i : importManager.imports»
			import «i»;
		«ENDFOR»
		«body»
	'''
	/* Compile the abstract class body */
	def compileAbstractClassBody(ObjectDeclaration object, ImportManager im)'''

		«val name = abstractClassName(object.name)»		
		«val interfaces = getDeclaredInteractionInterfaces(object.name)»
		
		/* Class «name» */
		@SuppressWarnings("all")
		«IF interfaces.empty »
			public abstract class «name» {
		«ELSE»
			public abstract class «name» implements «FOR i : interfaces SEPARATOR ','»«i»«ENDFOR» {
		«ENDIF»
			protected «interfaceName(object.name)» self;
			/* Default values */
			
			protected void initialize(final «object.name» «object.name.toLowerCase») {
				
			}
			«compileAbstractClassVirtualProperties(object,im)»			
			«compileAbstractClassMethods(object,im)»
		}	
	'''
	/* Declares convenient add and remove methods, including validation */
	def compileConvenientCollectionMethods(ObjectDeclaration object, ImportManager im) '''
		«FOR p: object.properties»
			«IF !"virtual".equalsIgnoreCase(p.virtual)»
				«IF Utils.isSupportedCollection( p.typeOrCollection.name)»
					«IF Utils.isPermissionGranted( p.permissions, "write" )»
					/** Convenient method to add «p.type.name». @param toAdd */
					public final void addTo«p.name.toFirstUpper»(final «p.type.name» toAdd) {
						«compileInvariants(object)»
						«compilePreConditionsAddTo(object,p,im)»
						self.get«p.name.toFirstUpper»().add(toAdd);
						«compileInvariants(object)»						
						«compilePostConditionsAddTo(object,p,im)»
					}
					«ENDIF»	
					«IF Utils.isPermissionGranted( p.permissions, "delete" )»
					/** Convenient method to remove «p.type.name». @param toRemove */
					public final void removeFrom«p.name.toFirstUpper»(final «p.type.name» toRemove) {
						«compileInvariants(object)»
						«compilePreConditionsRemoveFrom(object,p,im)»
						self.get«p.name.toFirstUpper»().remove(toRemove);
						«compileInvariants(object)»						
						«compilePostConditionsRemoveFrom(object,p,im)»
					}
					«ENDIF»
				«ENDIF»						
			«ENDIF»
		«ENDFOR»
	'''
	
	/* Compiles the abstract class methods */
	def compileAbstractClassMethods(ObjectDeclaration object, ImportManager im) '''
		«FOR m : object.methods»
			/* Method «m.name» */
			«IF !m.interactions.empty»
			@Override
			«ENDIF»
			public «m.returnType.name» «m.name»(«FOR p : m.parameters SEPARATOR ','»«p.type.name» «p.name»«ENDFOR») {
				throw new UnsupportedOperationException("TODO: provide implementation");
			}			
		«ENDFOR»
	'''

	/* Compile the abstract class virtual properties */
	def compileAbstractClassVirtualProperties(ObjectDeclaration object, ImportManager im)'''
		«FOR p : object.properties»
			«IF Utils.isSupportedCollection( p.typeOrCollection.name )»
				«/* Declares Virtual properties */»
				«IF "virtual".equalsIgnoreCase( p.virtual )»
					@org.codehaus.jackson.annotate.JsonIgnore
					/* Virtual property «p.name» */
					public «p.typeOrCollection.name»<«p.type.name»> get«p.name.toFirstUpper»() {
						return null;
					}
				«ENDIF»	
			«ELSE»
				«/* Declares Virtual properties */»
				«IF "virtual".equalsIgnoreCase( p.virtual )»
					«compilePropertyAllow(object,p,"read")»
					/* Virtual property «p.name» */
					public «p.typeOrCollection.name» get«p.name.toFirstUpper»() {
						return null;
					}
				«ENDIF»						
			«ENDIF»
		«ENDFOR»
	'''
	
	
		/* Compile the domain class virtual properties */
	def compileDomainClassVirtualProperties(ObjectDeclaration object, ImportManager im)'''
		«FOR p : object.properties»
			«IF Utils.isSupportedCollection( p.typeOrCollection.name )»
				«/* Declares Virtual properties */»
				«IF "virtual".equalsIgnoreCase( p.virtual )»
					/** Virtual collection «p.name». 
					 *  @return a «p.typeOrCollection.name» of «p.type.name» 
					 */
					@org.codehaus.jackson.annotate.JsonIgnore
					public final «p.typeOrCollection.name»<«p.type.name»> get«p.name.toFirstUpper»() {
						«compileInvariants(object)»
						«compilePostConditionsVirtual(object,p,im)»
						return super.get«p.name.toFirstUpper»();
					}
				«ENDIF»	
			«ELSE»
				«/* Declares Virtual properties */»
				«IF "virtual".equalsIgnoreCase( p.virtual )»
					/** Virtual Property «p.name». 
					 *  @return a «p.typeOrCollection.name» 
					 */
					«compilePropertyAllow(object,p,"read")»
					public final «p.typeOrCollection.name» get«p.name.toFirstUpper»() {
						«compileInvariants(object)»
						«compilePostConditionsVirtual(object,p,im)»
						return super.get«p.name.toFirstUpper»();
					}
				«ENDIF»
			«ENDIF»
		«ENDFOR»
	'''
	
	

	/* Compile the interface  */
	def compileInterface(ObjectDeclaration object,String targetPackage)'''
		«log.debug("compiling: " + interfaceName(object.name) )»	
		«val name = interfaceName(object.name)»
		«val importManager = new ImportManager(true,Utils.createJvmType( name, targetPackage ))»
		
		«val body = compileInterfaceBody(object,importManager)»
		package «targetPackage»;

		import java.util.*;	
		import java.math.*;	
		«FOR i : importManager.imports»
			import «i»;
		«ENDFOR»
		
		«body»

	'''	
	/* Compile the interface body  */	
	def compileInterfaceBody(ObjectDeclaration object, ImportManager importManager) '''
		«val name = interfaceName(object.name)»
		/** Interface «name». */
		@SuppressWarnings("all")
		public interface «name» {
			«compileInterfaceMethods(object,importManager)»
		}	
	'''
	/* Compile the interface methods */
	def compileInterfaceMethods(ObjectDeclaration object, ImportManager im) '''
		/** Returns id property. @return id */
		Long getId();
		/** Set the id property. @param id */
		void setId(Long id);	
		«FOR p : object.properties»			
			«IF Utils.isSupportedCollection( p.typeOrCollection.name )»
				«/* Virtual properties are not declared in the interface */»
				«IF !"virtual".equalsIgnoreCase( p.virtual )»
					/** Returns the «p.name» collection. @return «p.name» */
					«p.typeOrCollection.name»<«p.type.name»> get«p.name.toFirstUpper»();
					/** Sets the «p.name» collection. @param collection «p.typeOrCollection.name» of «p.type.name» */
					void set«p.name.toFirstUpper»(«p.typeOrCollection.name»<«p.type.name»> collection);
				«ENDIF»	
			«ELSE»
				«/* Virtual properties are not declared in the interface */»
				«IF !"virtual".equalsIgnoreCase( p.virtual )»
					/** Returns the «p.name» property. @return «p.name» */
					«p.typeOrCollection.name» get«p.name.toFirstUpper»();
					/** Set the «p.name» property. @param «p.name» */
					void set«p.name.toFirstUpper»(«p.typeOrCollection.name» «p.name»);
				«ENDIF»						
			«ENDIF»
		«ENDFOR»
	'''
	/* Read Permission  */
	def compilePropertyAllow(ObjectDeclaration object, PropertyDeclaration property,String right)'''		
		«log.debug("compiling " + right + " permissions for: " + object.name + " -> " + property.name +  Utils.dump(" visibility", property.visibility ) + " " + Utils.dumpPermissions(property.permissions) )»
		«IF  "hidden".equalsIgnoreCase(property.visibility)»
			@org.codehaus.jackson.annotate.JsonIgnore		
		«ELSEIF !Utils.isPermissionGranted(property.permissions, right)»
			@org.codehaus.jackson.annotate.JsonIgnore
		«ELSE»
			@org.codehaus.jackson.annotate.JsonProperty	
		«ENDIF»
	'''
	/* Returns the list of writable properties */
	def getWriteableProperties(EList<PropertyDeclaration> properties) {
		var writable = new ArrayList<PropertyDeclaration>(); 
		for(p : properties) {
			if( isWritableProperty( p ) ) {
				writable.add( p )	
			}
		}
		return writable
	}
	
	def isWritableProperty(PropertyDeclaration property) {
		/* Collections are not writable */
		if( Utils.isSupportedCollection(property.typeOrCollection.name))
			return false
			
		/* Virtual properties are not writable */	
		if( "virtual".equals(property.virtual) ) 			
			return false
			
		/* if no permission is specified is writable */	
		if( property.permissions.empty ) 			
			return true
			
		/* if has write permission is writable */	
		for(p:property.permissions) {
			if( "write".equalsIgnoreCase( p.name ))
				return true
		}			
			
			
		return false;
	}
	
	
	/* Computes the interface class name */
	def interfaceName(String className) {
		return className + "Interface"
	}
	/* Computes the abstract class name */
	def abstractClassName(String className) {
		return className + "Impl"
	}
	/* Computes the data transfer object class name */
	def dtoClassName(String className) {
		return className + "DTO"
	}	
	/* compute the target path */	 
	def pathName(String pkg) {
		return pkg.replace(".","/")
	}
	def getIndex() {
		this.index = this.index + 1;
		return index 
	}	
	
	def interactionInterfaceName(String className,String interaction) {
		return className + "$" + interaction
	}
	
	def contractClassName(String className) {
		return className + "$Contract"
	}
	
		/* Retrieve the list on interaction interfaces from the symboltTable */
	def getDeclaredInteractionInterfaces(String className) {
		var declaredInterfaces = new ArrayList() 
		for(i : symbolTable.interactionInterfaces.keySet() ) {
			if( i.startsWith( className + "$" ) ) {
				declaredInterfaces.add( i )
			}
		}
		return declaredInterfaces
	}

}