package com.mylaensys.dsl.generator.pojo

import com.mylaensys.dsl.generator.ServiceGenerator
import com.mylaensys.dsl.gape.ModuleDeclaration
import com.mylaensys.dsl.gape.ObjectDeclaration
import org.eclipse.xtext.generator.IFileSystemAccess
import org.eclipse.xtext.xbase.compiler.ImportManager
import org.eclipse.xtext.common.types.TypesFactory
import com.mylaensys.dsl.generator.SymbolTable
import com.mylaensys.dsl.generator.utils.Utils
import com.mylaensys.dsl.gape.PropertyDeclaration
import org.eclipse.emf.common.util.EList
import java.util.ArrayList
import org.apache.log4j.Logger

class ServiceGeneratorPojo implements ServiceGenerator {
	protected static Logger log =  Logger.getLogger("ServiceGenerator")
	private int index = 0;
	protected SymbolTable symbolTable

	
	override compile(ModuleDeclaration module, ObjectDeclaration object,SymbolTable symbolTable, IFileSystemAccess fsa) {

		/* Compile the Service */
		this.symbolTable = symbolTable
		fsa.generateFile(pathName(module.name) + "/" + serviceName(object.name) + ".java", compileServiceClass(object,module.name));		
	}
	/* Compile Service Class */
	def compileServiceClass(ObjectDeclaration object, String targetPackage) '''
		«val importManager = new ImportManager(true,Utils.createJvmType( object.name, targetPackage ))»
		«val body = compileServiceClassBody(object,targetPackage,importManager)»
		package «targetPackage»;
		
		import java.util.*;
		«FOR i : importManager.imports»
			import «i»;
		«ENDFOR»
		«body»
	'''
	/* Service Class Body */
	def compileServiceClassBody(ObjectDeclaration object, String targetPackage, ImportManager im) '''
		«val name = serviceName(object.name)»	
		@javax.ws.rs.Path("/1.0/«Utils.makePlural(object.name.toLowerCase)»")
		public class «name» extends com.mylaensys.dsl.gape.AbstractService {
			«injectLogger(name,targetPackage,im)»

			«compileRegistry(object,im)»
			«compileMethodAll(object,targetPackage,im)»
			«compileMethodGet(object,targetPackage,im)»
			«compileMethodCreate(object,targetPackage,im)»
			«compileMethodUpdate(object,targetPackage,im)»
			«compileCollections(object,targetPackage,im)»
			
		}
	'''
	
	/* Generates the collections services  */
	def compileCollections(ObjectDeclaration object, String targetPackage, ImportManager im)'''
		«FOR p : object.properties»
			«IF Utils.isSupportedCollection( p.typeOrCollection.name )»
				«compileCollection(object,targetPackage,p,im)»
			«ENDIF»
		«ENDFOR»
	'''
	
	/* Compile a collection */
	def compileCollection(ObjectDeclaration object, String targetPackage, PropertyDeclaration property, ImportManager im)'''
		«log.debug("compiling collection for object: " + object.name + " -> " + property.name )»
		/* Virtual collections are read only*/ 
		«IF "virtual".equalsIgnoreCase( property.virtual )»
			«compileCollectionMethodGet(object,targetPackage,property,im)»
		«ELSE»
			«/* The collection must have the permissions "read" */»
			«IF Utils.isPermissionGranted( property.permissions , "read" )»
				«compileCollectionMethodGet(object,targetPackage,property,im)»
			«ENDIF»
			«/* The collection must have the permissions "write" */»
			«IF Utils.isPermissionGranted( property.permissions , "write" )»
				«compileCollectionMethodAdd(object,targetPackage,property,im)»		
				«compileCollectionMethodCreate(object,targetPackage,property,im)»
			«ENDIF»
			«/* The collection must have the permissions "delete" */»
			«IF Utils.isPermissionGranted( property.permissions , "delete" )»
				«compileCollectionMethodRemove(object,targetPackage,property,im)»
			«ENDIF»
		«ENDIF»
	'''
	
	/* Creates and add to the collection */
	def compileCollectionMethodCreate(ObjectDeclaration declaration, String string, PropertyDeclaration declaration2, ImportManager manager) '''
	'''
	
	
	def compileCollectionMethodGet(ObjectDeclaration object,String targetPackage, PropertyDeclaration property, ImportManager manager)'''
		«log.debug("compiling collection method GET for object: " + object.name + " -> " + property.name )»
		@javax.ws.rs.GET
		@javax.ws.rs.Path("/{id}/«property.name»")
		@javax.ws.rs.Produces("application/json")
		public javax.ws.rs.core.Response get«property.name.toFirstUpper»(@javax.ws.rs.PathParam("id") Long id,@javax.ws.rs.QueryParam("offset") Long offset,@javax.ws.rs.QueryParam("limit") Long limit) {
			
		}
	'''
	
	/* Compile a collection */
	def compileCollectionMethodAdd(ObjectDeclaration object, String targetPackage,PropertyDeclaration property, ImportManager im)'''
		«log.debug("compiling collection method ADD for object: " + object.name + " -> " + property.name )»
	''' 

	/* Compile a collection */
	def compileCollectionMethodRemove(ObjectDeclaration object,String targetPackage, PropertyDeclaration property, ImportManager im)'''
		«log.debug("compiling collection method REMOVE for object: " + object.name + " -> " + property.name )»
	''' 
	 
	
	/* Generates GET - /objects */
	def compileMethodAll(ObjectDeclaration object, String targetPackage, ImportManager im) '''
		@javax.ws.rs.GET
		@javax.ws.rs.Path("/")
		@javax.ws.rs.Produces("application/json")
		public javax.ws.rs.core.Response all(@javax.ws.rs.QueryParam("offset") Long offset,@javax.ws.rs.QueryParam("limit") Long limit) {
			«compileSecurity(object)»
			
			return javax.ws.rs.core.Response.status(400).build();
		}		
	'''
	
	/* Generates GET - /objects/{id} */
	def compileMethodGet(ObjectDeclaration object, String string2, ImportManager im) '''
		@javax.ws.rs.GET
		@javax.ws.rs.Path("/{id}")
		@javax.ws.rs.Produces("application/json")
		public javax.ws.rs.core.Response get(@javax.ws.rs.PathParam("id") Long id) {
			«compileSecurity(object)»
			
			return javax.ws.rs.core.Response.status(400).build();
		}		
	'''

	/* Generates POST - /objects */
	def compileMethodCreate(ObjectDeclaration object, String targetPackage, ImportManager im) '''
		«val dto = Utils.createJvmType( object.name + "DTO" , targetPackage )»
		«val clazz = Utils.createJvmType( object.name, targetPackage )»
		«val audit = Utils.createJvmType("AuditData","com.mylaensys.dsl.gape")»
		«val hashmap = Utils.createJvmType("HashMap","java.util")»
		@javax.ws.rs.POST
		@javax.ws.rs.Path("/")
		@javax.ws.rs.Consumes("application/json")
		@javax.ws.rs.Produces("application/json")
		public javax.ws.rs.core.Response create(«im.serialize(dto)» dto) {
			«compileSecurity(object)»
			
			try {
				«im.serialize(clazz)» toStore = new «im.serialize(clazz)»();
				«im.serialize(hashmap)»<String,«im.serialize(audit)»> changes = submit( dto , toStore );
				«im.serialize(hashmap)»<String, String> errors = validateProperties( toStore );
				if( errors.size() == 0 ) {
					 
					String url = javax.ws.rs.core.UriBuilder.fromResource (this.getClass()).path("/{id}").build(toStore.getId()).toString();
					java.net.URI location = new java.net.URI(url);
					return javax.ws.rs.core.Response.created(location).status(201).entity(toStore).build();
				} else {
					return javax.ws.rs.core.Response.status(400).entity(errors).build();
				}
				return javax.ws.rs.core.Response.status(200).entity(object).build();
			} catch (Exception e ) {
				«compileException(object)»
			} finally {
				if(txn.isActive()) {
					txn.rollback();
				}				
				pm.close();
			}
			return javax.ws.rs.core.Response.status(_status).entity(_message).build();				
		}		
	'''
	def compileMethodUpdate(ObjectDeclaration object, String targetPackage, ImportManager im) '''
	
	'''
	/* For subclasses */
	def compileRegistry(ObjectDeclaration object, ImportManager im) '''
		«val registry = Utils.createJvmType("RegistryObject","com.mylaensys.dsl.gape")»
		/** 
		* Returns the Registry.
		* @param name object name 
		* @return registry 
		*/
		@Override
		public final «im.serialize(registry)» getRegistryObject(final String name) {
			return  RegistryImpl.getInstance().getObject(name);
		}
	'''
	/* Declares the Logger */
	def injectLogger(String name, String targetPackage, ImportManager im) '''	
		«val logger = Utils.createJvmType("Logger","java.util.logging")»
		«val clazz = Utils.createJvmType(name ,targetPackage)»
		/** Logger. */
		private static final «im.serialize(logger)» LOG = Logger.getLogger(«im.serialize(clazz)».class.getName());
	'''	
	
	/* Perform security check */
	def compileSecurity(ObjectDeclaration declaration) '''
		/* Security check */
	'''
	
	/* Exception  */
	def compileException(ObjectDeclaration declaration)'''
		status = Response.Status.INTERNAL_SERVER_ERROR;
		message = e.getMessage();
	'''		
	def getIndex() {
		this.index = this.index + 1;
		return index 
	}	
	
	/* Computes the service name */
	def serviceName(String className) {
		return className + "Service"
	}	
	/* Compute the target path */	 
	def pathName(String pkg) {
		return pkg.replace(".","/")
	}
	/* Compute the version */
	def version(String ver) {
   		return "v" + ver.replace(".","") 
   	}
   
   
	
}