/*
 * generated by Xtext
 */
package com.mylaensys.dsl.validation

import org.eclipse.xtext.validation.Check
import com.mylaensys.dsl.gape.ObjectDeclaration
import com.mylaensys.dsl.gape.GapePackage
import com.mylaensys.dsl.gape.PropertyDeclaration
import com.mylaensys.dsl.gape.MethodDeclaration
import com.mylaensys.dsl.gape.ModuleDeclaration


import static extension org.eclipse.xtext.EcoreUtil2.*
import com.mylaensys.dsl.gape.InteractionDeclaration
import com.mylaensys.dsl.generator.utils.Utils

/**
 * Custom validation rules. 
 *
 * see http://www.eclipse.org/Xtext/documentation.html#validation
 */
class GapeValidator extends AbstractGapeValidator {


	public static val DUPLICATE_METHOD = 'duplicateMethodName'
	public static val DUPLICATE_OBJECT = 'duplicateObjectName'
	public static val DUPLICATE_PROPERTY = 'duplicatePropertyName'
	public static val DUPLICATE_INTERACTION = 'duplicateInteractionName'
	
	public static val INVALID_OBJECT_NAME = 'invalidObjectName'
	public static val INVALID_METHOD_NAME = 'invalidMethodName'
	public static val INVALID_PROPERTY_NAME = 'invalidPropertyName'
	public static val INVALID_INTERACTION_NAME = 'invalidInteractionName'
	public static val INVALID_INTERACTION_REFERENCE = 'invalidInteractionReference'	
	public static val UNSUPPORTED_COLLECTION = 'unsupportedCollection'	
	public static val INVALID_OBJECT_REFERENCE = 'invalidObjectRefence'
	
	
	
	
	
	@Check
	def checkObjectDeclarationStartsWithCapital(ObjectDeclaration object) {
		if (!Character.isUpperCase(object.name.charAt(0))) {
			warning('Object name should start with a capital', 
					GapePackage::eINSTANCE.objectDeclaration_Name,
					INVALID_OBJECT_NAME)
		}
	}
	
	@Check
	def checkPropertyDeclarationStartsWithLowercase(PropertyDeclaration property) {
		if (!Character.isLowerCase(property.name.charAt(0))) {
			warning('Property name should not start with a capital', 
					GapePackage::eINSTANCE.propertyDeclaration_Name,
					INVALID_PROPERTY_NAME)
		}
	}
	
	@Check
	def checkMethodDeclarationStartsWithLowercase(MethodDeclaration method) {
		if (!Character.isLowerCase(method.name.charAt(0))) {
			warning('Method name should not start with a capital', 
					GapePackage::eINSTANCE.methodDeclaration_Name,
					INVALID_METHOD_NAME)
		}
	}
	
		@Check
	def checkInteractionDeclarationStartsWithCapital(InteractionDeclaration interaction) {
		if (!Character.isUpperCase(interaction.name.charAt(0))) {
			warning('Interaction name should start with a capital', 
					GapePackage::eINSTANCE.interactionDeclaration_Name,
					INVALID_INTERACTION_NAME)
		}
	}
	
	
	
	@Check
	def void checkNoDuplicateObject(ObjectDeclaration object) {
		if( object.getContainerOfType(typeof(ModuleDeclaration)).objects.exists[
			it != object && it.name == object.name
		]) error("Duplicate object '" + object.name + "'",
				GapePackage::eINSTANCE.objectDeclaration_Name,
				DUPLICATE_OBJECT
			)
	}

	@Check
	def void checkNoDuplicateProperty(PropertyDeclaration property) {
		if( property.getContainerOfType(typeof(ObjectDeclaration)).properties.exists[
			it != property && it.name == property.name
		]) error("Duplicate property '" + property.name + "'",
				GapePackage::eINSTANCE.propertyDeclaration_Name,
				DUPLICATE_PROPERTY 
			)
			
		if( property.getContainerOfType(typeof(ObjectDeclaration)).methods.exists[
			it != property && it.name == property.name
		]) error("The object has a method with name '" + property.name + "'",
				GapePackage::eINSTANCE.propertyDeclaration_Name,
				DUPLICATE_PROPERTY 
			)
	}

	@Check
	def void checkNoDuplicateMethod(MethodDeclaration method) {
		if( method.getContainerOfType(typeof(ObjectDeclaration)).methods.exists[
			it != method && it.name == method.name
		]) error("Duplicate method '" + method.name + "'",
				GapePackage::eINSTANCE.methodDeclaration_Name,
				DUPLICATE_METHOD 
			)
			
		if( method.getContainerOfType(typeof(ObjectDeclaration)).properties.exists[
			it != method && it.name == method.name
		]) error("The object has a property with name '" + method.name + "'",
				GapePackage::eINSTANCE.methodDeclaration_Name,
				DUPLICATE_METHOD 
			)
	}
	
	@Check
	def void checkNoDuplicateInteraction(InteractionDeclaration interaction) {
		if( interaction.getContainerOfType(typeof(ObjectDeclaration)).interactions.exists[
			it != interaction && it.name == interaction.name
		]) error("Duplicate interaction '" + interaction.name + "'",
				GapePackage::eINSTANCE.interactionDeclaration_Name,
				DUPLICATE_INTERACTION 
			)
			
	}
	
	@Check
	def void checkMethodInteractionReferences(MethodDeclaration method) {
		if(!method.interactions.empty) {
			val object = method.getContainerOfType(typeof(ObjectDeclaration))
			val module = object.getContainerOfType(typeof(ModuleDeclaration))
			
			for(String name : method.interactions) {
				var found = false;
				if( method.getContainerOfType(typeof(ObjectDeclaration)).interactions.exists[
					it.name == name 
				] ) { found = true }
				
				 
				if( object.getContainerOfType(typeof(ModuleDeclaration)).interactions.exists[
					it.name == name 
				] ) { found = true }
					
				if(!found) {
					error("Invalid interaction '"+ name +"'",
						GapePackage::eINSTANCE.methodDeclaration_Interactions,
						INVALID_INTERACTION_REFERENCE
					)
				}
			}
		}
	}
	
	@Check
	def void checkCollectionLinkReferences(PropertyDeclaration property) {
		if(property.ref != null ) {				
			val object = property.getContainerOfType(typeof(ObjectDeclaration))
			var target = object.getContainerOfType(typeof(ModuleDeclaration)).objects.findFirst[ 
				it.name == property.targetRef.name
			]; 
		if( target != null )
			if( !target.getContainerOfType(typeof(ObjectDeclaration)).properties.exists[
				it.name == property.ref
			])
			error("Invalid reference, object '" + property.targetRef.name + "' does not have a '" + property.ref + "' property",
					GapePackage::eINSTANCE.propertyDeclaration_Ref,
					INVALID_OBJECT_REFERENCE
				)
			 
			
		}
	}
}
